clc;
clearvars;

%-------------------------------------------------------------------------
%Simulation Parameters (MPU-6500)
%-------------------------------------------------------------------------

%Accelerometer
digital_resolution = 16;
measurement_range = 2; %n*g
accel_odr = 100; %Hz

%Errors
noise_psd = 300 * 10^(-6); %ug/sqrt(Hz)
cross_axis_sensitivity = 0.02; %Percent
alignment_error = 0.02; %Percent

%Cube configuration (m)
L_x = 0.04;
L_y = 0.04;
L_z = 0.012; 

%-------------------------------------------------------------------------
%Data Import
%-------------------------------------------------------------------------

%filename = 'yaw_slow_to_fast_3_gyr.csv';
filename = 'mixed_motion_2_gyr.csv';
gyr = csvread(filename,1,0);

t1 = gyr(:,1);
dt = t1(5) - t1(4);
fs = 1/dt;
len = length(t1);

angular_vel_x = gyr(:,2);
angular_vel_y = gyr(:,3);
angular_vel_z = gyr(:,4);

%Low-pass filter
order = 1;
filt_cut_off = 1;
[b, a] = butter(order, (2*filt_cut_off)/fs,'low');

%{
angular_vel_x = filtfilt(b,a,angular_vel_x);
angular_vel_y = filtfilt(b,a,angular_vel_y);
angular_vel_z = filtfilt(b,a,angular_vel_z);
%}

figure('Name','Angular Velocity from Smartphone','NumberTitle','off','Color',[1 1 1]);
plot(t1,rad2deg(angular_vel_x), 'g');
xlabel('time (seconds)');
ylabel('rad/s');
hold on;
plot(t1,rad2deg(angular_vel_y), 'r');
hold on;
plot(t1,rad2deg(angular_vel_z), 'b');
legend('X','Y','Z','Location', 'eastoutside');
ylabel('deg/s');
xlabel('Time (seconds)');

%Integrate to find angle (used as a reference later on)
angular_x = zeros(1, length(angular_vel_x));
angular_y = zeros(1, length(angular_vel_y));
angular_z = zeros(1, length(angular_vel_z));

for i = 2:length(angular_vel_x)
   angular_x(i) = angular_x(i-1) + angular_vel_x(i)*dt; 
   angular_y(i) = angular_y(i-1) + angular_vel_y(i)*dt; 
   angular_z(i) = angular_z(i-1) + angular_vel_z(i)*dt; 
end

%Differentiate to find angular acceleration
angular_accel_x = diff(angular_vel_x)/dt;
angular_accel_x(length(angular_accel_x) + 1) = angular_accel_x(length(angular_accel_x)); 

angular_accel_y = diff(angular_vel_y)/dt;
angular_accel_y(length(angular_accel_y) + 1) = angular_accel_y(length(angular_accel_y)); 

angular_accel_z = diff(angular_vel_z)/dt;
angular_accel_z(length(angular_accel_z) + 1) = angular_accel_z(length(angular_accel_z)); 

%-------------------------------------------------------------------------
%FFT-Analysis
%-------------------------------------------------------------------------
%{
accel_fft = fft(angular_z);

P2 = abs(accel_fft/len);
P1 = P2(1:len/2+1);
P1(2:end-1) = 2*P1(2:end-1);

f = fs*(0:(len/2))/len;
figure('Name','FFT of measured acceleration','NumberTitle','off','Color',[1 1 1]);
plot(f,P1)
title('Single-Sided Amplitude Spectrum of Accel X1')
xlabel('f (Hz)')
ylabel('|P1(f)|')
%}
%-------------------------------------------------------------------------
%Calculate cube dimensions
%-------------------------------------------------------------------------

figure('Name','Cube Configuration','NumberTitle','off','Color',[1 1 1]);
plotcube([L_x L_y L_z],[ 0 0 0],.0,[1 1 1]); 
axis([-0.01 L_x+0.01 -0.01 L_y+0.01 -0.01 0.01+L_z]);
ylabel('m');
xlabel('m');
hold on;
plotcube([0.004 0.004 0.001],[ 0 L_y-0.004 0],.8,[1 0 0]); 
hold on;
plotcube([0.004 0.004 0.001],[ L_x-0.004 0 L_z],.8,[0 0 1]); 

d_yz = sqrt(L_y^2 + L_z^2);
d_xz = sqrt(L_x^2 + L_z^2);
d_xy = sqrt(L_x^2 + L_y^2);

disp(d_yz);
disp(d_xz);
disp(d_xy);

angle_yz = asin(L_y/d_yz);
angle_xz = asin(L_z/d_xz);
angle_xy = asin(L_x/d_xy);

disp(rad2deg(angle_yz));
disp(rad2deg(angle_xz));
disp(rad2deg(angle_xy));

%-------------------------------------------------------------------------
%Decompose angular acceleration to measured acceleration
%-------------------------------------------------------------------------

%Calculate tangential acceleration from angular acceleration and cube shape
tangential_accel_xy = (angular_accel_z * d_xy)/2;
tangential_accel_yz = (angular_accel_x * d_yz)/2;
tangential_accel_xz = (angular_accel_y * d_xz)/2;

%Calculate measured acceleration
accel_x = tangential_accel_xy / cos(angle_xy);
accel_y = tangential_accel_yz / cos(angle_yz);
accel_z = tangential_accel_xz / cos(angle_xz);

%Add cross-axis sensitivity error
accel_x = accel_x + cross_axis_sensitivity*(accel_y + accel_z);
accel_y = accel_y + cross_axis_sensitivity*(accel_x + accel_z);
accel_z = accel_z + cross_axis_sensitivity*(accel_x + accel_y);

%Decompose and decimate acceleration for both accelerometers
decimation_factor = 1;%floor(100/accel_odr);

measured_accel_x1 = decimate(accel_x,decimation_factor);
measured_accel_x2 = decimate(-accel_x,decimation_factor);

measured_accel_y1 = decimate(accel_y,decimation_factor);
measured_accel_y2 = decimate(-accel_y,decimation_factor);

measured_accel_z1 = decimate(accel_z,decimation_factor);
measured_accel_z2 = decimate(-accel_z,decimation_factor);

t2 = decimate(t1,decimation_factor);
dt2 = t2(5) - t2(4);
fs2 = 1/dt2
len2 = length(t2);

%Calculating SNR based on accel parameters
noise_rms = 4 * noise_psd * sqrt(fs2/2); %Pk-2-Pk RMS noise
noise_snr = 20*log(((2*measurement_range)/(2*sqrt(2)))/noise_rms);

%Add Gaussian white noise to the acceleration
measured_accel_x1 = awgn(measured_accel_x1,noise_snr,'measured');
measured_accel_x2 = awgn(measured_accel_x2,noise_snr,'measured');

measured_accel_y1 = awgn(measured_accel_y1,noise_snr,'measured');
measured_accel_y2 = awgn(measured_accel_y2,noise_snr,'measured');

measured_accel_z1 = awgn(measured_accel_z1,noise_snr,'measured');
measured_accel_z2 = awgn(measured_accel_z2,noise_snr,'measured');

enob = floor(((noise_snr - 1.76)/6.02));
sensitivity = (2*measurement_range*9.81)/(2^digital_resolution);
disp(enob);

%Quantization of data
measured_accel_x1 = floor(measured_accel_x1/sensitivity);
measured_accel_x1 = measured_accel_x1 * sensitivity;

measured_accel_x2 = floor(measured_accel_x2/sensitivity);
measured_accel_x2 = measured_accel_x2 * sensitivity;

measured_accel_y1 = floor(measured_accel_y1/sensitivity);
measured_accel_y1 = measured_accel_y1 * sensitivity;

measured_accel_y2 = floor(measured_accel_y2/sensitivity);
measured_accel_y2 = measured_accel_y2 * sensitivity;

measured_accel_z1 = floor(measured_accel_z1/sensitivity);
measured_accel_z1 = measured_accel_z1 * sensitivity;

measured_accel_z2 = floor(measured_accel_z2/sensitivity);
measured_accel_z2 = measured_accel_z2 * sensitivity;

%Clip accel values that are outside measurement range
measured_accel_x1(measured_accel_x1 > measurement_range*10) = measurement_range*10;
measured_accel_x1(measured_accel_x1 < -measurement_range*10) = -measurement_range*10;

measured_accel_x2(measured_accel_x2 > measurement_range*10) = measurement_range*10;
measured_accel_x2(measured_accel_x2 < -measurement_range*10) = -measurement_range*10;

measured_accel_y1(measured_accel_y1 > measurement_range*10) = measurement_range*10;
measured_accel_y1(measured_accel_y1 < -measurement_range*10) = -measurement_range*10;

measured_accel_y2(measured_accel_y2 > measurement_range*10) = measurement_range*10;
measured_accel_y2(measured_accel_y2 < -measurement_range*10) = -measurement_range*10;

measured_accel_z1(measured_accel_z1 > measurement_range*10) = measurement_range*10;
measured_accel_z1(measured_accel_z1 < -measurement_range*10) = -measurement_range*10;

measured_accel_z2(measured_accel_z2 > measurement_range*10) = measurement_range*10;
measured_accel_z2(measured_accel_z2 < -measurement_range*10) = -measurement_range*10;

%Plot measured X-acceleration
figure('Name','Linear X-acceleration','NumberTitle','off','Color',[1 1 1]);
plot(t2,measured_accel_x1,'b');
axis([0 inf -20 20]);
ylabel('Linear acceleration (m/s^2)');
xlabel('Time (seconds)');

%-------------------------------------------------------------------------
%Calculate angular rates
%-------------------------------------------------------------------------

%Decompose measured acceleration to tangential acceleration
tangential_accel_x1 = measured_accel_x1 * cos(angle_xy*(1-alignment_error));
tangential_accel_x2 = measured_accel_x2 * cos(angle_xy*(1+alignment_error));

tangential_accel_y1 = measured_accel_y1 * cos(angle_yz*(1-alignment_error));
tangential_accel_y2 = measured_accel_y2 * cos(angle_yz*(1+alignment_error));

tangential_accel_z1 = measured_accel_z1 * cos(angle_xz*(1-alignment_error));
tangential_accel_z2 = measured_accel_z2 * cos(angle_xz*(1+alignment_error));

%Calculate difference in tangential acceleration
a_dx = (tangential_accel_x1 - tangential_accel_x2)/2;
a_dy = (tangential_accel_y1 - tangential_accel_y2)/2;
a_dz = (tangential_accel_z1 - tangential_accel_z2)/2;

%Angular accleration normalized to unit radius
alpha_dx = (2*a_dy)/d_yz;
alpha_dy = (2*a_dz)/d_xz;
alpha_dz = (2*a_dx)/d_xy;

%Integrate to find angular velocity
omega_dx = zeros(1, length(alpha_dx));
omega_dy = zeros(1, length(alpha_dy));
omega_dz = zeros(1, length(alpha_dz));

for i = 2:length(alpha_dz)
   omega_dx(i) = omega_dx(i-1) + alpha_dx(i)*dt2; 
   omega_dy(i) = omega_dy(i-1) + alpha_dy(i)*dt2; 
   omega_dz(i) = omega_dz(i-1) + alpha_dz(i)*dt2; 
end

%Add initial condition
omega_dx = omega_dx + angular_vel_x(1);
omega_dy = omega_dy + angular_vel_y(1);
omega_dz = omega_dz + angular_vel_z(1);

%Integrate to find angle
tetha_dx = zeros(1, length(alpha_dx));
tetha_dy = zeros(1, length(alpha_dy));
tetha_dz = zeros(1, length(alpha_dz));

for i = 2:length(alpha_dz)
   tetha_dx(i) = tetha_dx(i-1) + omega_dx(i)*dt2; 
   tetha_dy(i) = tetha_dy(i-1) + omega_dy(i)*dt2; 
   tetha_dz(i) = tetha_dz(i-1) + omega_dz(i)*dt2; 
end

angular_vel_x = transpose(angular_vel_x);
angular_vel_y = transpose(angular_vel_y);
angular_vel_z = transpose(angular_vel_z);

disp(immse(omega_dx,angular_vel_x));
disp(immse(omega_dy,angular_vel_y));
disp(immse(omega_dz,angular_vel_z));

figure('Name','Angular velocity about the Z-Axis','NumberTitle','off','Color',[1 1 1]);
plot(t2,rad2deg(omega_dz),'r');
xlabel('Time (seconds)');
hold on;
plot(t1,rad2deg(angular_vel_z),'g');
legend('Accelerometers','Gyroscope');
ylabel('deg/s');
xlabel('Time (seconds)');

figure('Name','Angular velocity about the X-Axis','NumberTitle','off','Color',[1 1 1]);
plot(t2,rad2deg(omega_dx),'r');
xlabel('Time (seconds)');
hold on;
plot(t1,rad2deg(angular_vel_x),'g');
legend('Accelerometers','Gyroscope');
ylabel('deg/s');
xlabel('Time (seconds)');

figure('Name','Angular velocity about the Y-Axis','NumberTitle','off','Color',[1 1 1]);
plot(t2,rad2deg(omega_dy),'r');
xlabel('Time (seconds)');
hold on;
plot(t1,rad2deg(angular_vel_y),'g');
legend('Accelerometers','Gyroscope');
ylabel('deg/s');
xlabel('Time (seconds)');