#include <stdio.h>
#include <math.h>

//Nordic SDK
#include "boards.h"

#include "app_util_platform.h"
#include "app_uart.h"
#include "app_error.h"

#include "nrf_delay.h"
#include "nrf_drv_timer.h"
#include "nrf_drv_gpiote.h"

#include "i2cdev.h"
#include "mpu6500.h"

 typedef struct {
         int16_t x;
         int16_t y;
         int16_t z;
 } Axis3i16;

 typedef struct {
         int32_t x;
         int32_t y;
         int32_t z;
 } Axis3i32;

 typedef struct {
         float x;
         float y;
         float z;
 } Axis3f;

#define USE_TIMER_ISR_APPROACH 1
#define USE_DEFAULT_ECOIMU_EQUATION 0
 
#define IMU_NUMBER_OF_BIAS_SAMPLES 100
 
//Scale factor
#define IMU_DEG_PER_LSB_CFG MPU6500_DEG_PER_LSB_250
#define IMU_G_PER_LSB_CFG MPU6500_G_PER_LSB_2
#define IMU_1G_RAW (int16_t)(1.0f / MPU6500_G_PER_LSB_2)

//UART buffer size
#define UART_TX_BUF_SIZE 256
#define UART_RX_BUF_SIZE 1

#define M_PI 3.1415f

float alpha = 400.0f;
 
#define INT_IMU1 28
#define INT_IMU2 29
#define FSYNC1	 30
#define FSYNC2	 31

const nrf_drv_timer_t TIMER0 = NRF_DRV_TIMER_INSTANCE(0);

//Global variables
Axis3i16 accel16, gyro16;
Axis3i16 accel_bias_1;
Axis3i16 accel_bias_2;
Axis3i16 gyro_bias;
Axis3f accel_1;
Axis3f accel_2;
Axis3f gyro;

//Cube parameters
float L_x = 0.04f;
float L_y = 0.04f;
float L_z = 0.012f;

float d_yz = 0.0f;
float d_xz = 0.0f;
float d_xy = 0.0f;

float angle_yz = 0.0f;
float angle_xz = 0.0f;
float angle_xy = 0.0f;

float angle_yz_comp = 0.0f;
float angle_xz_comp = 0.0f;
float angle_xy_comp = 0.0f;

float a_d_xy = 0.0f;
float a_d_xz = 0.0f;
float a_d_yz = 0.0f;

float alpha_dx = 0.0f;
float alpha_dy = 0.0f;
float alpha_dz = 0.0f;

float omega_dx = 0.0f;
float omega_dy = 0.0f;
float omega_dz = 0.0f;

float tetha_dx = 0.0f;
float tetha_dy = 0.0f;
float tetha_dz = 0.0f;

float gravity_x1 = 0.0f, gravity_y1 = 0.0f, gravity_z1 = 0.0f;
float gravity_x2 = 0.0f, gravity_y2 = 0.0f, gravity_z2 = 0.0f;

float roll = 0.0f, pitch = 0.0f, yaw = 0.0f;

volatile bool mpu_interrupt_1 = false;
volatile bool mpu_interrupt_2 = false;

//Function declarations
void calculate_angular_velocity(float ax1, float ay1, float az1, float ax2, float ay2, float az2, float dt);
void calculate_gravity_roll_pitch(float ax1, float ay1, float az1, float ax2, float ay2, float az2);

void mpu6500_int_handler_1(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
	mpu_interrupt_1 = true;
}

void mpu6500_int_handler_2(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
	mpu_interrupt_2 = true;
}

static void uart_events_handler(app_uart_evt_t * p_event)
{
	switch (p_event->evt_type)
	{
		case APP_UART_COMMUNICATION_ERROR:
				APP_ERROR_HANDLER(p_event->data.error_communication);
				break;

		case APP_UART_FIFO_ERROR:
				APP_ERROR_HANDLER(p_event->data.error_code);
				break;

		default:
				break;
	}
}

static void uart_config(void)
{
	uint32_t err_code;
	const app_uart_comm_params_t comm_params = {
			RX_PIN_NUMBER,
			TX_PIN_NUMBER,
			RTS_PIN_NUMBER,
			CTS_PIN_NUMBER,
			APP_UART_FLOW_CONTROL_DISABLED,
			false,
			UART_BAUDRATE_BAUDRATE_Baud230400
	};

	APP_UART_FIFO_INIT(&comm_params,
										 UART_RX_BUF_SIZE,
										 UART_TX_BUF_SIZE,
										 uart_events_handler,
										 APP_IRQ_PRIORITY_LOW,
										 err_code);

	APP_ERROR_CHECK(err_code);
}

void timer_event_handler(nrf_timer_event_t event_type, void* p_context)
{
    switch(event_type)
    {
        case NRF_TIMER_EVENT_COMPARE0:
						//GPIO Used for timing measurements
						nrf_gpio_pin_set(4);

						i2cdev_select_bus(0);
						mpu6500SetAddress(MPU6500_ADDRESS_AD0_LOW);
						mpu6500GetAcceleration(&accel16.x,&accel16.y,&accel16.z);
						
						accel16.x -= accel_bias_1.x;
						accel16.y -= accel_bias_1.y;
						accel16.z -= accel_bias_1.z;
						accel16.z += IMU_1G_RAW;
						
						accel_1.x = ((float)accel16.x * IMU_G_PER_LSB_CFG)*9.807f;
						accel_1.y = ((float)accel16.y * IMU_G_PER_LSB_CFG)*9.807f;
						accel_1.z = ((float)accel16.z * IMU_G_PER_LSB_CFG)*9.807f;
						
						mpu6500SetAddress(MPU6500_ADDRESS_AD0_HIGH);
						mpu6500GetAcceleration(&accel16.x,&accel16.y,&accel16.z);
						
						accel16.x -= accel_bias_2.x;
						accel16.y -= accel_bias_2.y;
						accel16.z -= accel_bias_2.z;
						accel16.z += IMU_1G_RAW;
						
						accel_2.x = ((float)accel16.x * IMU_G_PER_LSB_CFG)*9.807f;
						accel_2.y = ((float)accel16.y * IMU_G_PER_LSB_CFG)*9.807f;
						accel_2.z = ((float)accel16.z * IMU_G_PER_LSB_CFG)*9.807f;
						
						i2cdev_select_bus(1);
						mpu6500SetAddress(MPU6500_ADDRESS_AD0_LOW);
						mpu6500GetRotation(&gyro16.x, &gyro16.y, &gyro16.z);

						gyro16.x -= gyro_bias.x;
						gyro16.y -= gyro_bias.y;
						gyro16.z -= gyro_bias.z;
						
						gyro.x = (float)gyro16.x * IMU_DEG_PER_LSB_CFG;
						gyro.y = (float)gyro16.y * IMU_DEG_PER_LSB_CFG;
						gyro.z = (float)gyro16.z * IMU_DEG_PER_LSB_CFG;
						
						calculate_angular_velocity(accel_1.x,accel_1.y,accel_1.z,accel_2.x,accel_2.y,accel_2.z,0.01);
						
						//GPIO Used for timing measurements
						nrf_gpio_pin_clear(4);
						
						//printf("%.3f,%.3f,%.3f,%.3f, %.3f,%.3f,%.3f,%.3f,%.3f \r\n",accel_1.x,accel_1.y,accel_1.z,accel_2.x,accel_2.y,accel_2.z,gyro.x,gyro.y,gyro.z);
						//printf("%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f \r\n",omega_dx,omega_dy,omega_dz,alpha,alpha_dy,alpha_dz,gyro.x,gyro.y,gyro.z);
            break;
        
        default:
            //Do nothing.
            break;
    }    
}

void timer_config(void)
{
		uint32_t time_ms = 10; //Time(in miliseconds) between consecutive compare events.
		uint32_t time_ticks;
		uint32_t err_code = NRF_SUCCESS;

		err_code = nrf_drv_timer_init(&TIMER0, NULL, timer_event_handler);
		APP_ERROR_CHECK(err_code);

		time_ticks = nrf_drv_timer_ms_to_ticks(&TIMER0, time_ms);

		nrf_drv_timer_extended_compare(&TIMER0, NRF_TIMER_CC_CHANNEL0, time_ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);
}

void attatch_intterupt_1(void)
{
    ret_code_t err_code;

    err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);
    
    nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_HITOLO(true);
    in_config.pull = NRF_GPIO_PIN_PULLUP;

    err_code = nrf_drv_gpiote_in_init(INT_IMU1, &in_config, mpu6500_int_handler_1);
    APP_ERROR_CHECK(err_code);

    nrf_drv_gpiote_in_event_enable(INT_IMU1, true);
}

void attatch_intterupt_2(void)
{
	ret_code_t err_code;

	//err_code = nrf_drv_gpiote_init();
	APP_ERROR_CHECK(err_code);

	nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_HITOLO(true);
	in_config.pull = NRF_GPIO_PIN_PULLUP;

	err_code = nrf_drv_gpiote_in_init(INT_IMU2, &in_config, mpu6500_int_handler_2);
	APP_ERROR_CHECK(err_code);

	nrf_drv_gpiote_in_event_enable(INT_IMU2, true);
}

void calculate_accel_bias(int16_t *ax_mean, int16_t *ay_mean, int16_t *az_mean)
{
	int32_t ax_temp = 0, ay_temp = 0, az_temp = 0;

	//Discard 50 first samples
	for(uint16_t i = 0; i < 50; i++) {
			mpu6500GetAcceleration(&accel16.x,&accel16.y,&accel16.z);
	}

	//Find bias
	for(uint16_t i = 0; i < IMU_NUMBER_OF_BIAS_SAMPLES; i++) {
			mpu6500GetAcceleration(&accel16.x,&accel16.y,&accel16.z);
			ax_temp += accel16.x;
			ay_temp += accel16.y;
			az_temp += accel16.z;
	}

	*ax_mean = (int16_t)(ax_temp/IMU_NUMBER_OF_BIAS_SAMPLES);
	*ay_mean = (int16_t)(ay_temp/IMU_NUMBER_OF_BIAS_SAMPLES);
	*az_mean = (int16_t)(az_temp/IMU_NUMBER_OF_BIAS_SAMPLES);
}

void calculate_gyro_bias(int16_t *gx_mean, int16_t *gy_mean, int16_t *gz_mean)
{
	int32_t gx_temp = 0, gy_temp = 0, gz_temp = 0;

	//Discard 50 first samples
	for(uint16_t i = 0; i < 50; i++) {
			mpu6500GetRotation(&gyro16.x,&gyro16.y,&gyro16.z);
	}

	//Find bias
	for(uint16_t i = 0; i < IMU_NUMBER_OF_BIAS_SAMPLES; i++) {
			mpu6500GetRotation(&gyro16.x,&gyro16.y,&gyro16.z);
			gx_temp += gyro16.x;
			gy_temp += gyro16.y;
			gz_temp += gyro16.z;
	}

	*gx_mean = (int16_t)(gx_temp/IMU_NUMBER_OF_BIAS_SAMPLES);
	*gy_mean = (int16_t)(gy_temp/IMU_NUMBER_OF_BIAS_SAMPLES);
	*gz_mean = (int16_t)(gz_temp/IMU_NUMBER_OF_BIAS_SAMPLES);
}

void calculate_cube_dimensions(void) 
{
	d_yz = sqrtf(L_y*L_y + L_z*L_z); //d_x
	d_xz = sqrtf(L_x*L_x + L_z*L_z); //d_y
	d_xy = sqrtf(L_x*L_x + L_y*L_y); //d_z
	
	if(USE_DEFAULT_ECOIMU_EQUATION) {
		angle_yz = asinf(L_y/d_yz); //angle_y
		angle_xz = asinf(L_z/d_xz); //angle_z
		angle_xy = asinf(L_x/d_xy); //angle_x
	}
	else {
		angle_yz = asinf(L_y/d_yz); //angle_y
		angle_xz = asinf(L_z/d_xz); //angle_z
		angle_xy = asinf(L_x/d_xy); //angle_x
		
		angle_yz_comp = asinf(L_z/d_yz); //angle_y
		angle_xz_comp = asinf(L_x/d_xz); //angle_z
		angle_xy_comp = asinf(L_y/d_xy); //angle_x
	}
}

void calculate_gravity_roll_pitch(float ax1, float ay1, float az1, float ax2, float ay2, float az2)
{
	//Take average of roll and pitch estimates
	roll = atan2f(ay1,sqrtf(ax1*ax1 + az1*az1));
	pitch = atan2f(-ax1,sqrtf(ay1*ay1 + az1*az1));
	
	roll += atan2f(ay2,sqrtf(ax2*ax2 + az2*az2));
	pitch += atan2f(-ax2,sqrtf(ay2*ay2 + az2*az2));
	
	roll /= 2;
	pitch /= 2;
}

void calculate_angular_velocity(float ax1, float ay1, float az1, float ax2, float ay2, float az2, float dt) 
{
	//XY plane
	float tangential_accel_xy_y1;
	float tangential_accel_xy_y2;
	
	float tangential_accel_xy_x1;
	float tangential_accel_xy_x2;
	
	float tangential_accel_xy_dy;
	float tangential_accel_xy_dx;
	
	//XZ plane
	float tangential_accel_xz_z1;
	float tangential_accel_xz_z2;
	
	float tangential_accel_xz_x1;
	float tangential_accel_xz_x2;
	
	float tangential_accel_xz_dz;
	float tangential_accel_xz_dx;
	
	//YZ plane
	float tangential_accel_yz_z1;
	float tangential_accel_yz_z2;
	
	float tangential_accel_yz_y1;
  float tangential_accel_yz_y2;
	
	float tangential_accel_yz_dz;
	float tangential_accel_yz_dy;
	
	//Calculate roll and pitch from gravity
	calculate_gravity_roll_pitch(ax1,ay1,az1,ax2,ay2,az2);
	
	//Conversion between EcoIMU notation
	//d_yz <-> d_x
	//d_xz <-> d_y
	//d_xy <-> d_z
	
	//angle_xy <-> angle_x
	//angle_yz <-> angle_y
	//angle_xz <-> angle_z
	
	//XY plane
	tangential_accel_xy_y1 = ay1 * cosf(angle_xy);
	tangential_accel_xy_y2 = ay2 * cosf(angle_xy);
	
	tangential_accel_xy_x1 = ax1 * cosf(angle_xy);
	tangential_accel_xy_x2 = ax2 * cosf(angle_xy);
	
	tangential_accel_xy_dy = (tangential_accel_xy_y1 - tangential_accel_xy_y2)/2.0f;
	tangential_accel_xy_dx = (tangential_accel_xy_x1 - tangential_accel_xy_x2)/2.0f;
	
	//XZ plane
	tangential_accel_xz_z1 = az1 * cosf(angle_xz);
	tangential_accel_xz_z2 = az2 * cosf(angle_xz);
	
	tangential_accel_xz_x1 = ax1 * cosf(angle_xz);
	tangential_accel_xz_x2 = ax2 * cosf(angle_xz);
	
	tangential_accel_xz_dz = (tangential_accel_xz_z1 - tangential_accel_xz_z2)/2.0f;
	tangential_accel_xz_dx = (tangential_accel_xz_x1 - tangential_accel_xz_x2)/2.0f;
	
	//YZ plane
	tangential_accel_yz_z1 = az1 * cosf(angle_yz);
	tangential_accel_yz_z2 = az2 * cosf(angle_yz);
	
	tangential_accel_yz_y1 = ay1 * cosf(angle_yz);
	tangential_accel_yz_y2 = ay2 * cosf(angle_yz);
	
	tangential_accel_yz_dz = (tangential_accel_yz_z1 - tangential_accel_yz_z2)/2.0f;
	tangential_accel_yz_dy = (tangential_accel_yz_y1 - tangential_accel_yz_y2)/2.0f;
	
	if(USE_DEFAULT_ECOIMU_EQUATION) {
		a_d_xy = tangential_accel_xy_dx;
		a_d_xz = tangential_accel_xz_dz;
		a_d_yz = tangential_accel_yz_dy;
	} else {
		a_d_xy = (tangential_accel_xy_dy - tangential_accel_xy_dx)/2.0f;
		a_d_xz = (tangential_accel_xz_dx - tangential_accel_xz_dz)/2.0f;
		a_d_yz = (tangential_accel_yz_dz - tangential_accel_yz_dy)/2.0f;
	}
	
	alpha_dx = (2*a_d_yz)/d_yz;
	alpha_dy = (2*a_d_xz)/d_xz;
	alpha_dz = (2*a_d_xy)/d_xy;
	
	//If angular acceleration is zero over time, reset angular velocity estimate
	static uint32_t counter = 0;
	if((fabsf(alpha_dz)+fabsf(alpha_dz)+fabsf(alpha_dz))/3 < (1.4f/(cosf(roll)*cosf(pitch)))) {
		counter++;
		if(counter > (dt)*alpha) {
			omega_dz *= 0.65f;
			omega_dx *= 0.65f;
			omega_dy *= 0.65f;
		}
	} else {
			counter = 0;
	}

	//Integrate to find angular velocity
	omega_dx = omega_dx + alpha_dx*dt;
	omega_dy = omega_dy + alpha_dy*dt;
	omega_dz = omega_dz + alpha_dz*dt;
	
	//Integrate to find angle of rotation
	tetha_dx = tetha_dx + omega_dx*dt;
	tetha_dy = tetha_dy + omega_dy*dt;
	tetha_dz = tetha_dz + omega_dz*dt;
}

static void power_manage(void)
{
    // Wait for event.
    __WFE();

    // Clear Event Register.
    __SEV();
    __WFE();
}

int main(void)
{		
		calculate_cube_dimensions();
		
		uart_config();
		printf("GF-IMU Testbed - By: Erlend Hestnes\r\n");
		
		if(USE_TIMER_ISR_APPROACH) {
			//I2C0
			i2cdev_select_bus(0);
			i2cdev_initialize_0();
			i2cdev_enable_0(true);
			printf("TWI #0 initialized\r\n");
			
			mpu6500SetAddress(MPU6500_ADDRESS_AD0_LOW);
			mpu6050_init_accel();
			printf("MPU6500 Accel #1 - initialized\r\n");
			calculate_accel_bias(&accel_bias_1.x,&accel_bias_1.y,&accel_bias_1.z);
			printf("MPU6500 Accel #1 bias - ax: %d, ay: %d, az: %d\r\n",accel_bias_1.x,accel_bias_1.y,accel_bias_1.z);
			
			mpu6500SetAddress(MPU6500_ADDRESS_AD0_HIGH);
			mpu6050_init_accel();
			printf("MPU6500 Accel #2 - initialized\r\n");
			calculate_accel_bias(&accel_bias_2.x,&accel_bias_2.y,&accel_bias_2.z);
			printf("MPU6500 Accel #2 bias - ax: %d, ay: %d, az: %d\r\n",accel_bias_2.x,accel_bias_2.y,accel_bias_2.z);
			
			//I2C1
			i2cdev_select_bus(1);
			i2cdev_initialize_1();
			i2cdev_enable_1(true);
			printf("TWI #1 initialized\r\n");
			
			mpu6500SetAddress(MPU6500_ADDRESS_AD0_LOW);
			mpu6050_init();
			printf("MPU6500 Gyro #1 - initialized\r\n");
			calculate_gyro_bias(&gyro_bias.x,&gyro_bias.y,&gyro_bias.z);
			printf("MPU6500 Gyro #1 bias - ax: %d, ay: %d, az: %d\r\n",gyro_bias.x,gyro_bias.y,gyro_bias.z);
			
			//Initiate Timer ISR
			timer_config();
			nrf_drv_timer_enable(&TIMER0);
				
		} else {
			//I2C0
			i2cdev_select_bus(0);
			i2cdev_initialize_0();
			i2cdev_enable_0(true);
			printf("TWI #0 initialized\r\n");
			
			printf("1");
			mpu6500SetAddress(MPU6500_ADDRESS_AD0_LOW);
			printf("2");
			attatch_intterupt_1();
			printf("3");
			mpu6500_lp_accel_mode_init();
			printf("4");
			printf("MPU6500 Accel #1 - LP initialized\r\n");
			
			mpu6500SetAddress(MPU6500_ADDRESS_AD0_HIGH);
			attatch_intterupt_2();
			mpu6500_lp_accel_mode_init();
			printf("MPU6500 Accel #2 - LP initialized\r\n");

			//Enables LP accel mode
			mpu6500SetAddress(MPU6500_ADDRESS_AD0_LOW);
			mpu6500SetWakeCycleEnabled(true);
			mpu6500SetAddress(MPU6500_ADDRESS_AD0_HIGH);
			mpu6500SetWakeCycleEnabled(true);
			
			i2cdev_select_bus(1);
			i2cdev_initialize_1();
			i2cdev_enable_1(true);
			printf("TWI #1 initialized\r\n");
			
			mpu6500SetAddress(MPU6500_ADDRESS_AD0_LOW);
			mpu6050_init();
			printf("MPU6500 Gyro #1 - initialized\r\n");
			calculate_gyro_bias(&gyro_bias.x,&gyro_bias.y,&gyro_bias.z);
			printf("MPU6500 Gyro #1 bias - ax: %d, ay: %d, az: %d\r\n",gyro_bias.x,gyro_bias.y,gyro_bias.z);
			
		}
		
		//GPIO for run-time measurements
		nrf_gpio_cfg_output(4);
		
		uint32_t counter = 0;
		
		int16_t ax_mean1 = 0;
		int16_t ay_mean1 = 0;
		int16_t az_mean1 = 0;
		
		int16_t ax_mean2 = 0;
		int16_t ay_mean2 = 0;
		int16_t az_mean2 = 0;
		
		int32_t ax_temp1 = 0, ay_temp1 = 0, az_temp1 = 0;
		int32_t ax_temp2 = 0, ay_temp2 = 0, az_temp2 = 0;
		
    while(true) {	
			if(!USE_TIMER_ISR_APPROACH) {	
				while((!mpu_interrupt_1 || mpu_interrupt_2)) {
					power_manage();
				}
				__disable_irq();
				
				if(counter < IMU_NUMBER_OF_BIAS_SAMPLES) {
					i2cdev_select_bus(0);
					
					mpu6500SetAddress(MPU6500_ADDRESS_AD0_LOW);
					mpu6500GetAcceleration(&accel16.x,&accel16.y,&accel16.z);
					ax_temp1 += accel16.x;
					ay_temp1 += accel16.y;
					az_temp1 += accel16.z;
					
					mpu6500SetAddress(MPU6500_ADDRESS_AD0_HIGH);
					mpu6500GetAcceleration(&accel16.x,&accel16.y,&accel16.z);
					ax_temp2 += accel16.x;
					ay_temp2 += accel16.y;
					az_temp2 += accel16.z;
					
					counter++;
				} 
				else if (counter == IMU_NUMBER_OF_BIAS_SAMPLES) {
					ax_mean1 = (int16_t)(ax_temp1/IMU_NUMBER_OF_BIAS_SAMPLES);
					ay_mean1 = (int16_t)(ay_temp1/IMU_NUMBER_OF_BIAS_SAMPLES);
					az_mean1 = (int16_t)(az_temp1/IMU_NUMBER_OF_BIAS_SAMPLES);
					
					ax_mean2 = (int16_t)(ax_temp2/IMU_NUMBER_OF_BIAS_SAMPLES);
					ay_mean2 = (int16_t)(ay_temp2/IMU_NUMBER_OF_BIAS_SAMPLES);
					az_mean2 = (int16_t)(az_temp2/IMU_NUMBER_OF_BIAS_SAMPLES);
					
					printf("ax1: %d, ay1: %d, az1: %d\r\n", ax_mean1, ay_mean1, az_mean1);
					printf("ax2: %d, ay2: %d, az2: %d\r\n", ax_mean2, ay_mean2, az_mean2);
					
					counter++;
				} 
				else {
					//Used for timing measurements
					nrf_gpio_pin_set(4);
					
					i2cdev_select_bus(0);
					mpu6500SetAddress(MPU6500_ADDRESS_AD0_LOW);
					mpu6500GetAcceleration(&accel16.x,&accel16.y,&accel16.z);
					
					accel16.x -= ax_mean1;
					accel16.y -= ay_mean1;
					accel16.z -= az_mean1;
					accel16.z += IMU_1G_RAW;
					
					accel_1.x = ((float)accel16.x * IMU_G_PER_LSB_CFG)*9.807f;
					accel_1.y = ((float)accel16.y * IMU_G_PER_LSB_CFG)*9.807f;
					accel_1.z = ((float)accel16.z * IMU_G_PER_LSB_CFG)*9.807f;
					
					mpu6500SetAddress(MPU6500_ADDRESS_AD0_HIGH);
					mpu6500GetAcceleration(&accel16.x,&accel16.y,&accel16.z);
					
					accel16.x -= ax_mean2;
					accel16.y -= ay_mean2;
					accel16.z -= az_mean2;
					accel16.z += IMU_1G_RAW;
					
					accel_2.x = ((float)accel16.x * IMU_G_PER_LSB_CFG)*9.807f;
					accel_2.y = ((float)accel16.y * IMU_G_PER_LSB_CFG)*9.807f;
					accel_2.z = ((float)accel16.z * IMU_G_PER_LSB_CFG)*9.807f;
					
					i2cdev_select_bus(1);
					mpu6500SetAddress(MPU6500_ADDRESS_AD0_LOW);
					mpu6500GetRotation(&gyro16.x, &gyro16.y, &gyro16.z);

					gyro16.x -= gyro_bias.x;
					gyro16.y -= gyro_bias.y;
					gyro16.z -= gyro_bias.z;
					
					gyro.x = (float)gyro16.x * IMU_DEG_PER_LSB_CFG;
					gyro.y = (float)gyro16.y * IMU_DEG_PER_LSB_CFG;
					gyro.z = (float)gyro16.z * IMU_DEG_PER_LSB_CFG;
					
					calculate_angular_velocity(accel_1.x,accel_1.y,accel_1.z,accel_2.x,accel_2.y,accel_2.z,0.016);
					
					printf("%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f \r\n",omega_dx,omega_dy,omega_dz,alpha,alpha_dy,alpha_dz,gyro.x,gyro.y,gyro.z);
					
					//Used for timing measurements
					nrf_gpio_pin_clear(4);
				}
				mpu_interrupt_1 = false;
				mpu_interrupt_2 = false;
				__enable_irq();
		} 
		else {
			power_manage();
		}
  }
}
